import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { datatableRequest } from "@server/controllers/guild/api/dt.module";

export const get = async (request: Request, response: Response) => {
    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const filter = await prisma.filter.findUnique({
        where: { id: Number(request.query.id) }
    });

    if (!filter) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(filter);
};

export const post = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const { name, value, matching_algorithm, is_insensitive, is_whitelist } = request.body;

    let filter;

    try {
        filter = await prisma.filter.create({
            data: { 
                name: name,
                guild_id: guildId,
                value: value,
                matching_algorithm: matching_algorithm,
                is_insensitive: is_insensitive,
                is_whitelist: is_whitelist
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(filter);
};

export const patch = async () => {}  // TODO ...

export const del = async (request: Request, response: Response) => {
    const { ids } = request.body;
    const guildId = request.params.guildId;

    if (!ids || !Array.isArray(ids)) {
        response.status(400).json({ error: "invalid request body" });
        return;
    }

    try {
        await prisma.filter.deleteMany({ where: {
            id: { in: ids },
            guild_id: guildId
        }});
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(204).json(null);
};

export const datatable = async (request: Request, response: Response) => {
    return await datatableRequest(
        request,
        response,
        prisma.filter,
        { id: "asc" }
    );
};

export default { get, post, patch, del, datatable };